package router

import (
	"downloader/internal/app/api/download"
	"downloader/internal/core/ports/download_request"
	"downloader/pkg/common/uuid"
	"github.com/go-chi/chi"
	"github.com/go-chi/chi/middleware"
	"net/http"
	"time"
)

type router struct {
	internalRouter *chi.Mux
}

func NewRouter() *router {
	router := &router{internalRouter: chi.NewRouter()}

	router.
		setupMiddleware().
		setupRoutes()

	return router
}

func (router *router) Run() {
	_ = http.ListenAndServe(":3333", router.internalRouter)
}

func (router *router) RegisterApi() *chi.Mux {
	return router.internalRouter
}

func (router *router) setupMiddleware() *router {
	router.internalRouter.Use(middleware.Logger)
	router.internalRouter.Use(middleware.RequestID)
	router.internalRouter.Use(middleware.RealIP)
	router.internalRouter.Use(middleware.Recoverer)
	router.internalRouter.Use(middleware.Timeout(time.Second * 60))

	return router
}

func (router *router) setupRoutes() *router {
	setupDownloadRoute(router)

	return router
}

func setupDownloadRoute(router *router) {
	drRepository := download_request.NewInMemoryRepository()
	drBackgroundService := download_request.NewLocalBackgroundService(drRepository)
	gen := uuid.New()

	drService := download_request.NewLocalService(drRepository, gen, drBackgroundService)
	downloadApi := download.New(drService)
	downloadApi.SetupDownloadApi(router.internalRouter)
}
